from flask import Flask, request, jsonify
from admin_service.functions import get_token

application = Flask(__name__)
flint_authentication = ""


class App:

    def __init__(self):
        self.authentication = ""

    def register_authentication(self, authentication):
        global flint_authentication
        flint_authentication = authentication

    @staticmethod
    @application.route('/', methods=['POST'])
    def user():
        missing_args = []
        data = request.get_json()
        user_info = data.get("data", None)
        user_id = data.get("user", None)
        if user_info is None:
            missing_args.append("data")
        if user_id is None:
            missing_args.append("user")
        if len(missing_args) > 0:
            message = "Flint Admin Service API Missing params: {}".format(', '.join(missing_args))
            response = {
                "message": message,
                "status": "failure"
            }
        else:
            result = flint_authentication(**user_info)
            if result:
                token = get_token(user_id)
                response = {
                    "user": user_id,
                    "token": token,
                    "message": "",
                    "status": "success"
                }
            else:
                message = "User {} is not authenticated".format(user_id)
                response = {
                    "user": user_id,
                    "message": message,
                    "status": "failure"
                }
        return jsonify(response)

    @staticmethod
    def start():
        application.run(host='0.0.0.0', port='5000', debug=True)


def create_app():
    return App()



from kubernetes import client, config
from kubernetes.client.rest import ApiException
import base64
import time

config.load_kube_config()
api_instance = client.CoreV1Api()


class NameSpace:
    def __init__(self):
        pass

    def get(self, name):
        try:
            api_response = api_instance.read_namespace(name=name)
            return api_response
        except ApiException as e:
            status = e.status
            reason = e.reason
            reason = "Failed to get namespace {}.\n Reason: {}".format(name, reason)
            raise AdminServiceException(status=status, reason=reason)

    def create(self, name):
        try:
            body = {
                'kind': 'Namespace',
                'apiVersion': 'v1',
                'metadata': {
                    'name': name,
                }
            }
            api_response = api_instance.create_namespace(body=body)
        except ApiException as e:
            status = e.status
            reason = e.reason
            reason = "Failed to get namespace {}.\n Reason: {}".format(name, reason)
            raise AdminServiceException(status=status, reason=reason)


class ServiceAccount:
    def __init__(self):
        pass

    def get(self, name):
        try:
            api_response = api_instance.read_namespaced_service_account(name=name, namespace=name)
            return api_response
        except ApiException as e:
            status = e.status
            reason = e.reason
            reason = "Failed to get namespace {}.\n Reason: {}".format(name, reason)
            raise AdminServiceException(status=status, reason=reason)

    def create(self, name):
        try:
            body = {
                'kind': 'ServiceAccount',
                'apiVersion': 'v1',
                'metadata': {
                    'name': name,
                    'namespace': name
                },
            }
            api_response = api_instance.create_namespaced_service_account(namespace=name, body=body)
        except ApiException as e:
            status = e.status
            reason = e.reason
            reason = "Failed to get namespace {}.\n Reason: {}".format(name, reason)
            raise AdminServiceException(status=status, reason=reason)


class Secret:
    def __init__(self):
        pass

    def get(self, name, namespace):
        try:
            api_response = api_instance.read_namespaced_secret(name=name, namespace=namespace)
            return api_response
        except ApiException as e:
            status = e.status
            reason = e.reason
            reason = "Failed to get namespace {}.\n Reason: {}".format(name, reason)
            raise AdminServiceException(status=status, reason=reason)


def get_token(user):
    ns_helper = NameSpace()
    sa_helper = ServiceAccount()
    secret_helper = Secret()
    try:
        ns_helper.get(user)
    except AdminServiceException as e:
        if e.status == 404:
            ns_helper.create(user)
        else:
            raise e
    try:
        sa_helper.get(user)
    except AdminServiceException as e:
        if e.status == 404:
            sa_helper.create(user)
        else:
            raise e
    retry = 10
    i = 0
    while True:
        sa_obj = sa_helper.get(user)
        secrets = sa_obj.secrets
        print(secrets)
        if i >= retry:
            break
        if secrets is None:
            time.sleep(0.5)
            i += 1
            continue
        else:
            break
    secrets = sa_obj.secrets
    if secrets is None:
        reason = "Failed to get token for user {}".format(user)
        raise AdminServiceException(status=1001, reason=reason)
    secret = secrets[0].name
    secret_obj = secret_helper.get(secret, user)
    secret_token = 'Bearer ' + base64.b64decode(secret_obj.data["token"]).decode("utf-8")
    print(secret_token)
    return secret_token


class AdminServiceException(Exception):

    def __init__(self, status=None, reason=None):
        self.status = status
        self.reason = reason

    def __str__(self):
        error_message = "Reason: {0}\n".format(self.reason)
        return error_message




from setuptools import setup
import os
import sys

_here = os.path.abspath(os.path.dirname(__file__))

if sys.version_info[0] < 3:
    with open(os.path.join(_here, 'README.md')) as f:
        long_description = f.read()
else:
    with open(os.path.join(_here, 'README.md'), encoding='utf-8') as f:
        long_description = f.read()

version = {}
with open(os.path.join(_here, 'admin_service', 'version.py')) as f:
    exec(f.read(), version)

setup(
    name='flint-admin-service',
    version=version['__version__'],
    description=('flint-admin-service'),
    long_description=long_description,
    author='Gaoxin Dai',
    author_email='daigx1990@gmail.com',
    url='https://github.com/flintdev/flint-admin-service',
    license='Apache License 2.0',
    packages=['admin_service'],
    install_requires=[
        'flask==1.1.2',
        'kubernetes==11.0.0',
    ],
    include_package_data=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3'
    ]
    )
